library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;


library proc_common_v4_0;
use proc_common_v4_0.all;

library unisim;
use unisim.vcomponents.all;

entity vivado_npi_rd_v1_0_M00_AXI is
	generic (
		-- Users to add parameters here
        c_family                : string               := "virtex6";
        c_fifo_size             : integer              := 1024*2;

		-- User parameters ends
		-- Do not modify the parameters beyond this line

		-- Base address of targeted slave
		C_M_TARGET_SLAVE_BASE_ADDR	: std_logic_vector	:= x"40000000";
		-- Burst Length. Supports 1, 2, 4, 8, 16, 32, 64, 128, 256 burst lengths
		C_M_AXI_BURST_LEN	: integer	:= 16;
		-- Thread ID Width
		C_M_AXI_ID_WIDTH	: integer	:= 1;
		-- Width of Address Bus
		C_M_AXI_ADDR_WIDTH	: integer	:= 32;
		-- Width of Data Bus
		C_M_AXI_DATA_WIDTH	: integer	:= 32;
		-- Width of User Write Address Bus
		C_M_AXI_AWUSER_WIDTH	: integer	:= 1;
		-- Width of User Read Address Bus
		C_M_AXI_ARUSER_WIDTH	: integer	:= 1;
		-- Width of User Write Data Bus
		C_M_AXI_WUSER_WIDTH	: integer	:= 1;
		-- Width of User Read Data Bus
		C_M_AXI_RUSER_WIDTH	: integer	:= 1;
		-- Width of User Response Bus
		C_M_AXI_BUSER_WIDTH	: integer	:= 1
	);
	port (
		-- Users to add ports here
        cmd             : in std_logic_vector(7 downto 0);
        rd_fifo_clk     : in std_logic;
        rd_fifo_rd_en   : in std_logic;
        c_cnt_burst     : in std_logic_vector(11 downto 0);
        c_cnt_repeat    : in std_logic_vector(15 downto 0);
        rd_fifo_data    : out std_logic_vector(31 downto 0); 
        rd_npi_ready    : out std_logic;
        rd_fifo_empty   : out std_logic;
        rd_fifo_full    : out std_logic;
        hw_ver          : out std_logic_vector(31 downto 0);
        rd_npi_status0  : out std_logic_vector(31 downto 0);
        rd_npi_status1  : out std_logic_vector(31 downto 0);
        rd_npi_status2  : out std_logic_vector(31 downto 0);

		-- User ports ends
		-- Do not modify the ports beyond this line

		-- Global Clock Signal.
		M_AXI_ACLK	: in std_logic;
		-- Global Reset Singal. This Signal is Active Low
		M_AXI_ARESETN	: in std_logic;
		-- Master Interface Write Address ID
		M_AXI_AWID	: out std_logic_vector(C_M_AXI_ID_WIDTH-1 downto 0);
		-- Master Interface Write Address
		M_AXI_AWADDR	: out std_logic_vector(C_M_AXI_ADDR_WIDTH-1 downto 0);
		-- Burst length. The burst length gives the exact number of transfers in a burst
		M_AXI_AWLEN	: out std_logic_vector(7 downto 0);
		-- Burst size. This signal indicates the size of each transfer in the burst
		M_AXI_AWSIZE	: out std_logic_vector(2 downto 0);
		-- Burst type. The burst type and the size information, 
    -- determine how the address for each transfer within the burst is calculated.
		M_AXI_AWBURST	: out std_logic_vector(1 downto 0);
		-- Lock type. Provides additional information about the
    -- atomic characteristics of the transfer.
		M_AXI_AWLOCK	: out std_logic;
		-- Memory type. This signal indicates how transactions
    -- are required to progress through a system.
		M_AXI_AWCACHE	: out std_logic_vector(3 downto 0);
		-- Protection type. This signal indicates the privilege
    -- and security level of the transaction, and whether
    -- the transaction is a data access or an instruction access.
		M_AXI_AWPROT	: out std_logic_vector(2 downto 0);
		-- Quality of Service, QoS identifier sent for each write transaction.
		M_AXI_AWQOS	: out std_logic_vector(3 downto 0);
		-- Optional User-defined signal in the write address channel.
		M_AXI_AWUSER	: out std_logic_vector(C_M_AXI_AWUSER_WIDTH-1 downto 0);
		-- Write address valid. This signal indicates that
    -- the channel is signaling valid write address and control information.
		M_AXI_AWVALID	: out std_logic;
		-- Write address ready. This signal indicates that
    -- the slave is ready to accept an address and associated control signals
		M_AXI_AWREADY	: in std_logic;
		-- Master Interface Write Data.
		M_AXI_WDATA	: out std_logic_vector(C_M_AXI_DATA_WIDTH-1 downto 0);
		-- Write strobes. This signal indicates which byte
    -- lanes hold valid data. There is one write strobe
    -- bit for each eight bits of the write data bus.
		M_AXI_WSTRB	: out std_logic_vector(C_M_AXI_DATA_WIDTH/8-1 downto 0);
		-- Write last. This signal indicates the last transfer in a write burst.
		M_AXI_WLAST	: out std_logic;
		-- Optional User-defined signal in the write data channel.
		M_AXI_WUSER	: out std_logic_vector(C_M_AXI_WUSER_WIDTH-1 downto 0);
		-- Write valid. This signal indicates that valid write
    -- data and strobes are available
		M_AXI_WVALID	: out std_logic;
		-- Write ready. This signal indicates that the slave
    -- can accept the write data.
		M_AXI_WREADY	: in std_logic;
		-- Master Interface Write Response.
		M_AXI_BID	: in std_logic_vector(C_M_AXI_ID_WIDTH-1 downto 0);
		-- Write response. This signal indicates the status of the write transaction.
		M_AXI_BRESP	: in std_logic_vector(1 downto 0);
		-- Optional User-defined signal in the write response channel
		M_AXI_BUSER	: in std_logic_vector(C_M_AXI_BUSER_WIDTH-1 downto 0);
		-- Write response valid. This signal indicates that the
    -- channel is signaling a valid write response.
		M_AXI_BVALID	: in std_logic;
		-- Response ready. This signal indicates that the master
    -- can accept a write response.
		M_AXI_BREADY	: out std_logic;
		-- Master Interface Read Address.
		M_AXI_ARID	: out std_logic_vector(C_M_AXI_ID_WIDTH-1 downto 0);
		-- Read address. This signal indicates the initial
    -- address of a read burst transaction.
		M_AXI_ARADDR	: out std_logic_vector(C_M_AXI_ADDR_WIDTH-1 downto 0);
		-- Burst length. The burst length gives the exact number of transfers in a burst
		M_AXI_ARLEN	: out std_logic_vector(7 downto 0);
		-- Burst size. This signal indicates the size of each transfer in the burst
		M_AXI_ARSIZE	: out std_logic_vector(2 downto 0);
		-- Burst type. The burst type and the size information, 
    -- determine how the address for each transfer within the burst is calculated.
		M_AXI_ARBURST	: out std_logic_vector(1 downto 0);
		-- Lock type. Provides additional information about the
    -- atomic characteristics of the transfer.
		M_AXI_ARLOCK	: out std_logic;
		-- Memory type. This signal indicates how transactions
    -- are required to progress through a system.
		M_AXI_ARCACHE	: out std_logic_vector(3 downto 0);
		-- Protection type. This signal indicates the privilege
    -- and security level of the transaction, and whether
    -- the transaction is a data access or an instruction access.
		M_AXI_ARPROT	: out std_logic_vector(2 downto 0);
		-- Quality of Service, QoS identifier sent for each read transaction
		M_AXI_ARQOS	: out std_logic_vector(3 downto 0);
		-- Optional User-defined signal in the read address channel.
		M_AXI_ARUSER	: out std_logic_vector(C_M_AXI_ARUSER_WIDTH-1 downto 0);
		-- Write address valid. This signal indicates that
    -- the channel is signaling valid read address and control information
		M_AXI_ARVALID	: out std_logic;
		-- Read address ready. This signal indicates that
    -- the slave is ready to accept an address and associated control signals
		M_AXI_ARREADY	: in std_logic;
		-- Read ID tag. This signal is the identification tag
    -- for the read data group of signals generated by the slave.
		M_AXI_RID	: in std_logic_vector(C_M_AXI_ID_WIDTH-1 downto 0);
		-- Master Read Data
		M_AXI_RDATA	: in std_logic_vector(C_M_AXI_DATA_WIDTH-1 downto 0);
		-- Read response. This signal indicates the status of the read transfer
		M_AXI_RRESP	: in std_logic_vector(1 downto 0);
		-- Read last. This signal indicates the last transfer in a read burst
		M_AXI_RLAST	: in std_logic;
		-- Optional User-defined signal in the read address channel.
		M_AXI_RUSER	: in std_logic_vector(C_M_AXI_RUSER_WIDTH-1 downto 0);
		-- Read valid. This signal indicates that the channel
    -- is signaling the required read data.
		M_AXI_RVALID	: in std_logic;
		-- Read ready. This signal indicates that the master can
    -- accept the read data and response information.
		M_AXI_RREADY	: out std_logic
	);
end vivado_npi_rd_v1_0_M00_AXI;

architecture implementation of vivado_npi_rd_v1_0_m00_axi is
    signal axi_awvalid	: std_logic;
    signal axi_wdata	: std_logic_vector(c_m_axi_data_width-1 downto 0);
    signal axi_wlast	: std_logic;
    signal axi_wvalid	: std_logic;
    signal axi_bready	: std_logic;
    signal axi_araddr	: std_logic_vector(c_m_axi_addr_width-1 downto 0);
    signal axi_arvalid	: std_logic;
    signal axi_rready	: std_logic;

--	signal c_cnt_burst, cnt_burst : std_logic_vector(11 downto 0) := x"000";	-- save burst count
	signal cnt_burst : std_logic_vector(11 downto 0) := x"000";	-- save burst count
--	signal c_cnt_repeat, cnt_repeat: std_logic_vector(15 downto 0) := x"0000";	-- save how many repeat burst count, 0xffff means unlimit operation 
	signal cnt_repeat: std_logic_vector(15 downto 0) := x"0000";	-- save how many repeat burst count, 0xffff means unlimit operation 
	signal cnt_rx : std_logic_vector(3 downto 0) := x"0";	-- check whether rx, tx clock is operating
    signal step : std_logic_vector(7 downto 0);
    signal st_fifo : std_logic_vector(1 downto 0);
    signal rd_fifo_ready_s, rd_fifo_full_s, rd_fifo_empty_s, rd_fifo_rd_en_s, rd_fifo_rst_s  : std_logic; 
    signal rd_fifo_rd_count_s, rd_fifo_wr_count_s : std_logic_vector(2 downto 0);
    signal c_base_address : std_logic_vector(13 downto 0) := x"400" & "00" ;
    signal sm_reset_s, cmd_rd_start  : std_logic; 
--    signal rd_npi_status0 : std_logic_vector(31 downto 0);
--    signal rd_npi_status1 : std_logic_vector(31 downto 0);
--    signal rd_npi_status2 : std_logic_vector(31 downto 0);
--    signal hw_ver : std_logic_vector(31 downto 0) := x"00001000";

-----------------------    signal cmd : std_logic_vector(7 downto 0);
-----------------------    signal cnt_cmd : std_logic_vector(15 downto 0);
    component async_fifo_32kb
      port (
        rst : in std_logic;
        wr_clk : in std_logic;
        rd_clk : in std_logic;
        din : in std_logic_vector(31 downto 0);
        wr_en : in std_logic;
        rd_en : in std_logic;
        dout : out std_logic_vector(31 downto 0);
        full : out std_logic;
        empty : out std_logic;
        rd_data_count : out std_logic_vector(2 downto 0);
        wr_data_count : out std_logic_vector(2 downto 0)
      );
    end component;
--    attribute syn_black_box : boolean;
 --   attribute syn_black_box of async_fifo_32kb : component is true;
  --  attribute black_box_pad_pin : string;
   -- attribute black_box_pad_pin of async_fifo_32kb : component is "rst,wr_clk,rd_clk,din[31:0],wr_en,rd_en,dout[31:0],full,empty,rd_data_count[2:0],wr_data_count[2:0]";

begin


    --read address channel
    process(m_axi_aclk)										  
    begin                                                              
        if (rising_edge (m_axi_aclk)) then                               
            if (m_axi_aresetn = '0' or sm_reset_s = '1') then                                 
                step <= x"10";
                axi_arvalid <= '0';
                cnt_burst <= (others => '0'); 
                cnt_repeat <= (others => '0'); 
            else                                                           
                case step is
                    when x"10" =>
                        if cmd_rd_start = '1' then
                            if rd_fifo_wr_count_s(2) = '0' and rd_fifo_ready_s = '1' then 
                                axi_arvalid <= '1';
                                step <= x"20";
                            end if;
                        end if;
                    when x"20" =>
                        if m_axi_arready = '1' then 
                            axi_arvalid <= '0'; 
                            axi_rready <= '1'; 
                            step <= x"30";
                        end if;
                    when x"30" =>
                        if m_axi_rvalid = '1' then
                            if m_axi_rlast = '1' then
                                axi_rready <= '0';
                                step <= x"40";
                            end if;
                        end if;
                    when x"40" =>
                        if cnt_burst = (c_cnt_burst - 1) then
                            cnt_burst <= (others => '0');
                            if c_cnt_repeat /= x"ffff" then
                                if cnt_repeat = (c_cnt_repeat - 1) then
                                    cnt_repeat <= (others => '0');
                                    step <= x"00";
                                else
                                    cnt_repeat <= cnt_repeat + 1;
                                    step <= x"10";
                                end if;
                            else
                                step <= x"10";
                            end if;
                        else
                            cnt_burst <= cnt_burst + 1;
                            step <= x"10";
                        end if;
                    when others => null;
                end case;
            end if;
        end if;
    end process;

    process(m_axi_aclk)
    begin
        if m_axi_aclk'event and m_axi_aclk = '1' then
            if m_axi_aresetn = '0' or rd_fifo_rst_s = '1' then
                st_fifo <= "00";
                rd_fifo_ready_s <= '0';
            else
                case st_fifo is
                    when "00" =>
                        if rd_fifo_full_s = '0' and rd_fifo_empty_s = '1' then
                            rd_fifo_ready_s <= '1';
                            st_fifo <= "01";
                        end if;
                    when "01" =>
                        st_fifo <= "01";
                    when others => null;
                end case;
            end if;
        end if;
    end process;

	process(rd_fifo_clk)
	--process(m_axi_aclk)
	begin
		if rd_fifo_clk'event and rd_fifo_clk = '1' then
		--if m_axi_aclk'event and m_axi_aclk = '1' then
			cnt_rx <= cnt_rx + 1;
		end if;
	end process;


-- add user logic here
    process(m_axi_aclk)
    begin
        if m_axi_aclk'event and m_axi_aclk = '1' then
            if m_axi_aresetn = '0' then
                rd_fifo_rd_en_s <= '0'; 
            else
                if rd_fifo_rd_en_s = '1' then 
                    rd_fifo_rd_en_s <= '0'; 
                else
                    rd_fifo_rd_en_s <= '1'; 
                end if;
            end if;
        end if;
    end process;

    rd_npi_status0 <= x"000" & "000" & rd_fifo_rst_s & sm_reset_s & cmd_rd_start & cnt_rx & rd_fifo_full_s & rd_fifo_empty_s & step;
    rd_npi_status1 <= x"0" & cnt_burst   & cnt_repeat;  
    rd_npi_status2 <= x"0" & c_cnt_burst &  c_cnt_repeat;  

    i_rx_fifo : async_fifo_32KB
      PORT MAP (
        rst => rd_fifo_rst_s,
        wr_clk => m_axi_aclk,
        din => m_axi_rdata,
        wr_en => m_axi_rvalid,
        rd_clk => rd_fifo_clk,
        rd_en => rd_fifo_rd_en_s,
        dout => rd_fifo_data,
        full => rd_fifo_full_s,
        empty => rd_fifo_empty_s,
        rd_data_count => rd_fifo_rd_count_s,
        wr_data_count => rd_fifo_wr_count_s 
      );

--        din            => m_axi_rdata, 
--        wr_en          => m_axi_rvalid,
--        wr_clk         => m_axi_aclk,
--        rd_en          => rd_fifo_rd_en_s,
--        rd_clk         => m_axi_aclk,
--        ainit          => rd_fifo_rst_s,   
--        dout           => rd_fifo_data, 
--        full           => rd_fifo_full_s,
--        empty          => rd_fifo_empty_s,
--        almost_full    => open,
--        almost_empty   => open, 
--        wr_count       => rd_fifo_wr_count_s,
--        rd_count       => rd_fifo_rd_count_s,
--        rd_ack         => open,
--        rd_err         => open,
--        wr_ack         => open,
--        wr_err         => open);

--    i_rx_fifo: entity proc_common_v4_0.async_fifo_fg
--    generic map(
--        c_allow_2n_depth   => 1,  -- new paramter to leverage fifo gen 2**n depth
--        c_family           => c_family,  -- new for fifo gen
--        c_data_width       => 32,
--        c_enable_rlocs     => 0,  -- not supported in fg
--        c_fifo_depth       => c_fifo_size,                    
--        c_has_almost_empty => 0,
--        c_has_almost_full  => 0,
--        c_has_rd_ack       => 0,
--        c_has_rd_count     => 1,
--        c_has_rd_err       => 0,
--        c_has_wr_ack       => 0,
--        c_has_wr_count     => 1,
--        c_has_wr_err       => 0,
--        c_rd_ack_low       => 0,
--        c_rd_count_width   => 3,
--        c_rd_err_low       => 0,
--        c_use_blockmem     => 1,  -- 0 = distributed ram, 1 = bram
--        c_wr_ack_low       => 0,
--        c_wr_count_width   => 3,
--        c_wr_err_low       => 0   )
--    port map(
--        din            => m_axi_rdata, 
--        wr_en          => m_axi_rvalid,
--        wr_clk         => m_axi_aclk,
--        rd_en          => rd_fifo_rd_en_s,
--        rd_clk         => m_axi_aclk,
--        ainit          => rd_fifo_rst_s,   
--        dout           => rd_fifo_data, 
--        full           => rd_fifo_full_s,
--        empty          => rd_fifo_empty_s,
--        almost_full    => open,
--        almost_empty   => open, 
--        wr_count       => rd_fifo_wr_count_s,
--        rd_count       => rd_fifo_rd_count_s,
--        rd_ack         => open,
--        rd_err         => open,
--        wr_ack         => open,
--        wr_err         => open);
-- user logic ends
    rd_npi_ready <= cmd_rd_start;   
    rd_fifo_empty <= rd_fifo_empty_s;   
    rd_fifo_full <= rd_fifo_full_s;   
    rd_fifo_rd_en_s <= rd_fifo_rd_en;

    -------- default value -----------
    m_axi_awid	    <= (others => '0');
    m_axi_awaddr    <= (others => '0'); 
    m_axi_awlen	    <= x"00"; 
    m_axi_awsize	<= "000"; 
    m_axi_awburst	<= "01";
    m_axi_awlock	<= '0';
    m_axi_awcache	<= "0010";
    m_axi_awprot	<= "000";
    m_axi_awqos	    <= x"0";
    m_axi_awuser	<= (others => '1');
    m_axi_awvalid	<= axi_awvalid;
    m_axi_wdata	    <= axi_wdata;
    m_axi_wstrb	    <= (others => '1');
    m_axi_wlast	    <= axi_wlast;
    m_axi_wuser	    <= (others => '0');
    m_axi_wvalid	<= axi_wvalid;
    m_axi_bready	<= axi_bready;
    m_axi_arid	    <= (others => '0');
    m_axi_araddr	<= axi_araddr;
    m_axi_arlen	    <= x"0f";       -- 16 times 
    m_axi_arsize	<= "010";       -- 4 byte 
    m_axi_arburst	<= "01";
    m_axi_arlock	<= '0';
    m_axi_arcache	<= "0010";
    m_axi_arprot	<= "000";
    m_axi_arqos	    <= x"0";
    m_axi_aruser	<= (others => '1');
    m_axi_arvalid	<= axi_arvalid;
    m_axi_rready	<= axi_rready;

    axi_araddr <= c_base_address & cnt_burst & "00" & "0000";
    --write address channel
    axi_awvalid <= '0';                                            

    --write data channel
    axi_wvalid <= '0';                                                          
    axi_wlast <= '0';                                                           
    axi_wdata <= (others => '0'); 

    --write response (b) channel
    axi_bready <= '0';                                              

    ------- command emulation ----------------
    rd_fifo_rst_s <= '1' when cmd(7 downto 0) = x"89" else '0';
	sm_reset_s <= '1' when cmd(7 downto 0) = x"81" else '0'; 
    cmd_rd_start <= '1' when cmd(7 downto 0) = x"93" else '0';  

------------------    process(m_axi_aclk)										  
------------------    begin                                                              
------------------        if (rising_edge (m_axi_aclk)) then                               
------------------            if (m_axi_aresetn = '0') then
------------------                cnt_cmd <= x"0000";
------------------                cmd <= x"00";
------------------            else
------------------                cnt_cmd <= cnt_cmd + 1;
------------------                case cnt_cmd is
------------------                    when x"1000" =>
------------------                        cmd <= x"81";
------------------                    when x"1080" =>
------------------                        cmd <= x"00";
------------------                    when x"1100" =>
------------------                        cmd <= x"89";
------------------                    when x"1180" =>
------------------                        cmd <= x"00";
------------------                    when x"1500" =>
------------------                        c_cnt_burst <= x"004";
------------------                        c_cnt_repeat <= x"ffff";
------------------                    when x"1520" =>
------------------                        cmd <= x"93"; 
------------------                    when x"4000" =>
------------------                        cmd <= x"81";
------------------                    when x"4080" =>
------------------                        cmd <= x"00";
------------------                    when others =>
------------------                end case ;
------------------            end if;
------------------        end if;
------------------    end process;

end implementation;
